//+------------------------------------------------------------------+
//|                                               Moving Average.mq4 |
//|                      Copyright  2005, MetaQuotes Software Corp. |
//|                                       http://www.metaquotes.net/ |
//+------------------------------------------------------------------+
#define MAGICMA  20050610

extern double Lots               = 0.05;
extern double MaximumRisk        = 0.05;
extern double DecreaseFactor     = 3;
extern double MovingPeriod       = 14;
extern double MovingShift        = 2;
extern double MovingPeriod1       = 4;
extern double MovingShift1        = 2;
extern double Tral_Stop           =500;
//| Calculate open positions                                         |
//+------------------------------------------------------------------+
int CalculateCurrentOrders(string symbol)
  {
   int buys=0,sells=0;
//----
   for(int i=0;i<OrdersTotal();i++)
     {
      if(OrderSelect(i,SELECT_BY_POS,MODE_TRADES)==false) break;
      if(OrderSymbol()==Symbol() && OrderMagicNumber()==MAGICMA)
        {
         if(OrderType()==OP_BUY)  buys++;
         if(OrderType()==OP_SELL) sells++;
        }
     }
//---- return orders volume
   if(buys>0) return(buys);
   else       return(-sells);
  }
//+------------------------------------------------------------------+
//| Calculate optimal lot size                                       |
//+------------------------------------------------------------------+
double LotsOptimized()
  {
   double lot=Lots;
   int    orders=HistoryTotal();     // history orders total
   int    losses=0;                  // number of losses orders without a break
//---- select lot size
   lot=NormalizeDouble(AccountFreeMargin()*MaximumRisk/1000.0,1);
//---- calcuulate number of losses orders without a break
   if(DecreaseFactor>0)
     {
      for(int i=orders-1;i>=0;i--)
        {
         if(OrderSelect(i,SELECT_BY_POS,MODE_HISTORY)==false) { Print("Error in history!"); break; }
         if(OrderSymbol()!=Symbol() || OrderType()>OP_SELL) continue;
         //----
         if(OrderProfit()>0) break;
         if(OrderProfit()<0) losses++;
        }
      if(losses>1) lot=NormalizeDouble(lot-lot*losses/DecreaseFactor,1);
     }
//---- return lot size
   if(lot<0.1) lot=0.1;
   return(lot);
  }
//+------------------------------------------------------------------+
//| Check for open order conditions                                  |
//+------------------------------------------------------------------+
void CheckForOpen()
  {
   double ma;
   int    res;
//---- go trading only for first tiks of new bar
   if(Volume[0]>1) return;
//---- get Moving Average 
   ma=iMA(NULL,0,MovingPeriod,MovingShift,MODE_SMA,PRICE_CLOSE,0);
//---- sell conditions
   if(Open[1]>ma && Close[1]<ma)
     {
      res=OrderSend(Symbol(),OP_SELL,LotsOptimized(),Bid,3,0,0,"",MAGICMA,0,Red);
      return;
     }
   if(Open[1]<ma && Close[1]>ma)
     {
      res=OrderSend(Symbol(),OP_BUY,LotsOptimized(),Ask,3,0,0,"",MAGICMA,0,Blue);
      return;
     }

 }
//+------------------------------------------------------------------+
//| Check for close order conditions                                 |
//+------------------------------------------------------------------+
void CheckForClose()
  {
   double ma1;
//---- go trading only for first tiks of new bar
   if(Volume[0]>1) return;
//---- get Moving Average 
   ma1=iMA(NULL,0,MovingPeriod1,MovingShift1,MODE_SMA,PRICE_CLOSE,0);
//----
   for(int i=0;i<OrdersTotal();i++)
     {
      if(OrderSelect(i,SELECT_BY_POS,MODE_TRADES)==false)        break;
      if(OrderMagicNumber()!=MAGICMA || OrderSymbol()!=Symbol()) continue;
      //---- check order type 
      if(OrderType()==OP_BUY)
        {
         if(Open[1]>ma1 && Close[1]<ma1) OrderClose(OrderTicket(),OrderLots(),Bid,3,White);  ///Open[1]>ma && Close[1]<ma
         break;
        }
      if(OrderType()==OP_SELL)
        {
         RefreshRates();
         if(Open[1]<ma1 && Close[1]>ma1)OrderClose(OrderTicket(),OrderLots(),Ask,3,White);  //Open[1]<ma && Close[1]>ma
         break;
        }
        
     }

  } 
 
  void CheckForOrderModify()
  {
   string Symb=Symbol();                        // . 
//--------------------------------------------------------------- 2 --
   for(int i=1; i<=OrdersTotal(); i++)          //   
     {
      if (OrderSelect(i-1,SELECT_BY_POS)==true) //   
        {                                       //  :
         int Tip=OrderType();                   //  
         if(OrderSymbol()!=Symb||Tip>1)continue;//   
         double SL=OrderStopLoss();             // SL  .
         //------------------------------------------------------ 3 --
         while(true)                            //  
           {
            double TS=Tral_Stop;                //  
            int Min_Dist=MarketInfo(Symb,MODE_STOPLEVEL);//. 
            if (TS < Min_Dist)                  //   .
               TS=Min_Dist;                     //   TS
            //--------------------------------------------------- 4 --
            bool Modify=false;                  //    
            switch(Tip)                         //   
              {
               case 0 :                         //  Buy
                  if (NormalizeDouble(SL,Digits)< //   .
                     NormalizeDouble(Bid-TS*Point,Digits))
                    {
                     SL=Bid-TS*Point;           //   
                     string Text="Buy ";        //   Buy 
                     Modify=true;               //   .
                    }
                  break;                        //   switch
               case 1 :                         //  Sell
                  if (NormalizeDouble(SL,Digits)> //   .
                     NormalizeDouble(Ask+TS*Point,Digits)
                     || NormalizeDouble(SL,Digits)==0)//  
                    {
                     SL=Ask+TS*Point;           //   
                     Text="Sell ";              //   Sell 
                     Modify=true;               //   .
                    }
              }                                 //  switch
            if (Modify==false)                  //    
               break;                           //   while
            //--------------------------------------------------- 5 --
            double TP    =OrderTakeProfit();    // TP  .
            double Price =OrderOpenPrice();     //  . .
            int    Ticket=OrderTicket();        //  . .
 
            Alert (" ",Text,Ticket,".  ..");
            bool Ans=OrderModify(Ticket,Price,SL,TP,0);// !
            //--------------------------------------------------- 6 --
            if (Ans==true)                      //  :)
              {
               Alert (" ",Text,Ticket," :)");
               break;                           //   .
              }
            //--------------------------------------------------- 7 --
            int Error=GetLastError();           //   :(
            switch(Error)                       //  
              {
               case 130:Alert(" .   .");
                  RefreshRates();               //  
                  continue;                     //  . 
               case 136:Alert(" .   ..");
                  while(RefreshRates()==false)  //   
                     Sleep(1);                  //   
                  continue;                     //  . 
               case 146:Alert("  . ");
                  Sleep(500);                   //  
                  RefreshRates();               //  
                  continue;                     //  . 
                  //  
               case 2 : Alert(" .");
                  break;                        //   switch
               case 5 : Alert("   .");
                  break;                        //   switch
               case 64: Alert(" .");
                  break;                        //   switch
               case 133:Alert(" ");
                  break;                        //   switch
               default: Alert("  ",Error);//. 
              }
            break;                              //   .
           }                                    //   .
         //------------------------------------------------------ 8 --
        }                                       //   .
     }                                          //   .
//--------------------------------------------------------------- 9 --
   return;                                      //   start()
  }

//+------------------------------------------------------------------+
//| Start function                                                   |
//+------------------------------------------------------------------+
void start()
  {
//---- check for history and trading
   if(Bars<100 || IsTradeAllowed()==false) return;
//---- calculate open orders by current symbol
   if(CalculateCurrentOrders(Symbol())==0) 
   CheckForOpen();
   CheckForClose();
   CheckForOrderModify();
//----
  }
//+------------------------------------------------------------------+